# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2018-03-03 19:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('matchups', '0002_add_defaults_to_blank_fields'),
    ]

    operations = [
        migrations.AddField(
            model_name='matchup',
            name='featured',
            field=models.BooleanField(default=False),
        ),
    ]
