# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2018-03-03 19:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('matchups', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='matchup',
            name='description',
            field=models.TextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='matchup',
            name='email',
            field=models.EmailField(blank=True, db_index=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='matchup',
            name='period',
            field=models.CharField(blank=True, default='', max_length=32),
        ),
    ]
