# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2018-03-11 02:39
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('matchups', '0003_matchup_featured'),
    ]

    operations = [
        migrations.AlterField(
            model_name='matchup',
            name='period',
            field=models.CharField(blank=True, default='', max_length=32, validators=[django.core.validators.RegexValidator('^\\d+[dw]$')]),
        ),
    ]
