# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-05-05 01:11
from __future__ import unicode_literals

from django.db import migrations, models
import partial_index


class Migration(migrations.Migration):

    dependencies = [
        ('matchups', '0006_matchup_updated_on'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='matchup',
            index=models.Index(fields=['user', 'created_on'], name='matchups_ma_user_id_b8c76c_idx'),
        ),
        migrations.AddIndex(
            model_name='matchup',
            index=models.Index(fields=['user', 'updated_on'], name='matchups_ma_user_id_754211_idx'),
        ),
        migrations.AddIndex(
            model_name='matchup',
            index=partial_index.PartialIndex(fields=['created_on'], name='matchups_ma_created_250a2c_partial', unique=False, where='featured = true', where_postgresql='', where_sqlite=''),
        ),
        migrations.AddIndex(
            model_name='matchup',
            index=partial_index.PartialIndex(fields=['updated_on'], name='matchups_ma_updated_f84f32_partial', unique=False, where='featured = true', where_postgresql='', where_sqlite=''),
        ),
    ]
