# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-20 14:37
from __future__ import unicode_literals

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Matchup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(db_index=True, max_length=254, null=True)),
                ('title', models.CharField(db_index=True, max_length=1024)),
                ('slug', models.CharField(db_index=True, max_length=1024, unique=True)),
                ('description', models.TextField(null=True)),
                ('movies', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=128), size=None)),
                ('period', models.CharField(max_length=32, null=True)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]
