# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-25 12:03
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('movies', '0002_movie_title_not_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='PriceIndex',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('year', models.IntegerField(db_index=True)),
                ('month', models.IntegerField()),
                ('value', models.DecimalField(decimal_places=3, max_digits=12)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='priceindex',
            unique_together=set([('year', 'month')]),
        ),
    ]
