# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-05-10 10:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('matchups', '0007_matchup_datetime_indexes'),
    ]

    operations = [
        migrations.AddField(
            model_name='matchup',
            name='image',
            field=models.ImageField(null=True, upload_to='matchups/'),
        ),
    ]
